#!/usr/bin/python
# -*- coding: utf-8 -*-

import os, sys, optparse

# command line parser
parser = optparse.OptionParser()
parser.add_option('-p','--productname',
                  action='store',
                  dest='productname',
                  help='Name that will be given to the product')
parser.add_option('--skindir',
                  action='store',
                  dest='skindir',
                  help="Name that will be given to the folder in the product 'skins/' directory")
parser.add_option('--skinname',
                  action='store',
                  dest='skinname',
                  help="Name of the skin selection that will be added to 'portal_skins'")
parser.add_option('--stylesheet',
                  action='store',
                  dest='stylesheet',
                  help='Name of the future stylesheet file')
parser.add_option('-v',
                  action="store_true",
                  dest="verbose",
                  help="Verbose mode")

options,args = parser.parse_args()

productname = options.productname
skindir = options.skindir
skinname = options.skinname
stylesheet = options.stylesheet

path = os.path.dirname( os.path.abspath(__file__) )
if path.endswith('DIYPloneStyle%sbin' % os.sep):
    path = path[:-3]
else:
    raise IOError, 'This script must be in the DIYPloneStyle%sbin directory' % os.sep

if not productname or not os.path.isdir(path):
    parser.parse_args(args=['-h'])

if not skindir:
    skindir = '%s_styles' % productname.lower()

if not skinname:
    skinname = productname

if not stylesheet:
	stylesheet = '%s.css' % productname.lower()

if not stylesheet.endswith('.css'):
	stylesheet = '%s.css' % stylesheet

# mappings for substitutions
mapping = (
    ('DIYPloneStyle', productname),
    ('DIY Plone Style', skinname),
    ('renameThisFile.css', stylesheet),
   )

unused_lines = (
    '''# CHANGE 'DIYPloneStyle' to your product name in the following lines''',    
    '''{'id': 'diystylesheet.css', 'media': 'screen', 'rendering': 'import'},'''
   )

text_files = {
    'HISTORY.txt': '''Changelog for %s

    (name of developer listed in brackets)

%s 1.0 - Unreleased

    - Generated skeleton product from DIYPloneStyle.
      [generator]
''' % ( productname, productname ),
    'README.txt': '''Description

    %s is a product that adds a new style to a Plone 2.1.x portal.
    It creates a new skin selection to the 'portal_skins' tool
    (called %s), and registers a custom stylesheet (called %s)
    with the 'portal_css' tool.

    It is based on DIYPloneStyle, a skeleton product ready for building
    new graphical designs for Plone.

Installation

    Place %s in the Products directory of your Zope instance
    and restart the server.

    In Plone go to the 'Site Setup' page and click on the 'Add/Remove
    Products' link.

    Choose %s (check its checkbox) and click the 'Install' button.

    You may have to empty your browser cache to see the effects of the
    product installation/uninstallation.

    Uninstall -- This can be done from the same management screen.

Selecting a skin

   Depending on the value given to SELECTSKIN (in config.py), the skin will be
   selected (or not) as default one while installing the product. If you need
   to switch from a default skin to another, go to the 'Site Setup' page, and
   choose 'Skins' (as portal manager). You can also decide from that page if
   members can choose their preferred skin and, in that case, if the skin
   cookie should be persistent.

   Note -- Don't forget to perform a full refresh of the page or reload all
   images (not from browser cache) after selecting a skin.
   In Firefox, you can do so by pressing the 'shift' key while reloading the
   page. In IE, use the key combination <Ctrl-F5>.

Written by

    John Doe <john.doe@dev.null>
''' % ( productname, skinname, stylesheet, productname, productname ),
    'version.txt': '1.0',
   }

file_mapping = (
    ('renameThisFile.css.dtml','%s.dtml' % stylesheet),
   )


exts = ['.py','.dtml']

def log(message):
    if options.verbose:
        print >> sys.stdout, message

# rm example directory
def rmdir(dirname):
    log('Remove directory %s' % dirname)
    if os.path.isdir( dirname ):
        for filename in os.listdir(dirname):
            filepath = os.path.join(dirname, filename)
            if os.path.isfile( filepath ):
                os.remove( os.path.join(dirname, filename) )
            else:
                rmdir( filepath )
        os.rmdir( dirname )

rmdir( os.path.join(path,'skins','diystyle_example') )


# text substitution
def process(filepath):
    log('Processing %s' % filepath)
    f = open(filepath,'r')
    data = f.read()
    f.close()

    # remove unused lines
    output = []
    lines = data.split('\n')
    for line in lines:
        unused = False
        for unused_line in unused_lines:
            if line.find(unused_line) != -1:
                unused = True
                break
        if not unused:
            output.append(line)
    data = '\n'.join(output)
    
    # replace productname, skinname, etc.
    for orig, dest in mapping:
        data = data.replace(orig,dest)
    
    f = open(filepath,'w')
    f.write(data)
    f.close()

# substitution for text files
def process_txt(filepath, data):
    log('Processing %s' % filepath)
    f = open(filepath,'w')
    f.write(data)
    f.close()
    
# walk callback
def replace(args,dirname,filenames):
    if dirname in ['doc','docs',]:
        return
    if dirname.find('.svn') != -1:
        rmdir(dirname)
        return
    for filename in filenames:
        f,ext = os.path.splitext(filename)
        filepath = os.path.join(dirname,filename)
        if filename in text_files.keys():
            process_txt( filepath, text_files[filename] )
            continue
        if ext in ('.pyc','.pyo','.swp') or filename in ('.DS_Store',) \
           or filename.startswith('#') or filename.startswith('._'):
            os.remove( filepath )
        if filename == '.svn':
            rmdir(filepath)
        if ext not in exts:
            continue
        process( filepath )

os.path.walk(path,replace,None)


# rename files and directory
log('Rename files and directories')
skinpath = os.path.join(path,'skins','diystyle')

for orig, dest in file_mapping:
    orig = os.path.join(skinpath, orig)
    dest = os.path.join(skinpath, dest)
    if os.path.isfile(orig):
        os.rename(orig, dest)

if os.path.isdir(skinpath):
    os.rename( skinpath, os.path.join(path,'skins', skindir) )

# rm bin directory
log('Remove bin directory')
rmdir( os.path.join(path,'bin') )

log('Renaming DIYPloneStyle directory')
os.rename(path, path.replace('DIYPloneStyle',productname) )

print >> sys.stdout, '%s product generated successfully' % productname
